@@ -1,11 +1,11 @@
-#ifdef BSD
-#ifndef BSD4_1
-#define HAVE_GETPAGESIZE
-#endif
-#endif
+/* Emulate getpagesize on systems that lack it.  */
 
 #ifndef HAVE_GETPAGESIZE
 
+#ifdef VMS
+#define getpagesize() 512
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -25,10 +25,15 @@
 #define CLSIZE 1
 #endif /* no CLSIZE */
 #else /* no NBPG */
+#ifdef NBPC
 #define getpagesize() NBPC
+#else /* no NBPC */
+#ifdef PAGESIZE
+#define getpagesize() PAGESIZE
+#endif
+#endif /* NBPC */
 #endif /* no NBPG */
 #endif /* no EXEC_PAGESIZE */
 #endif /* no _SC_PAGESIZE */
 
 #endif /* not HAVE_GETPAGESIZE */
-
