@@ -1,5 +1,5 @@
 /* resuse.c - child process resource use library
-   Copyright (C) 1993 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,37 +13,30 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 /* Written by David MacKenzie, with help from
    arnej@imf.unit.no (Arne Henrik Juul)
    and pinard@iro.umontreal.ca (Francois Pinard).  */
 
-#include <sys/types.h>
-
+#include "wait.h"
 #include "port.h"
 
-#if HAVE_SYS_WAIT_H
-#include <sys/wait.h>
-#endif
-#ifndef WEXITSTATUS
-#define WEXITSTATUS(w) (((w) >> 8) & 0xff)
-#endif
-
 #if !HAVE_WAIT3
-#include <sys/times.h>
-#ifndef HZ
-#include <sys/param.h>
-#endif
-#if !defined(HZ) && defined(CLOCKS_PER_SEC)
-#define HZ CLOCKS_PER_SEC
-#endif
-#if !defined(HZ) && defined(CLK_TCK)
-#define HZ CLK_TCK
-#endif
-#ifndef HZ
-#define HZ 60
-#endif
+# include <sys/times.h>
+# ifndef HZ
+#  include <sys/param.h>
+# endif
+# if !defined(HZ) && defined(CLOCKS_PER_SEC)
+#  define HZ CLOCKS_PER_SEC
+# endif
+# if !defined(HZ) && defined(CLK_TCK)
+#  define HZ CLK_TCK
+# endif
+# ifndef HZ
+#  define HZ 60
+# endif
 #endif
 
 #include "resuse.h"
@@ -80,11 +73,7 @@ resuse_end (pid, resp)
      RESUSE *resp;
 #endif
 {
-#if HAVE_UNION_WAIT
-  union wait status;
-#else
   int status;
-#endif
 
 #if HAVE_WAIT3
   pid_t caught;
@@ -134,11 +123,7 @@ resuse_end (pid, resp)
     }
   resp->elapsed.tv_usec -= resp->start.tv_usec;
 
-#if HAVE_UNION_WAIT
-  resp->exitstatus = status.w_retcode;
-#else
-  resp->exitstatus = WEXITSTATUS (status);
-#endif
+  resp->waitstatus = status;
 
   return 1;
 }
