@@ -1,5 +1,5 @@
 /* `time' utility to display resource usage of processes.
-   Copyright (C) 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
+   Copyright (C) 1990, 91, 92, 93, 96 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,19 +13,18 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
-/* Originally written by David Keppel, pardo@cs.washington.edu.
-   Heavily modified by David MacKenzie, djm@gnu.ai.mit.edu.  */
+/* Originally written by David Keppel <pardo@cs.washington.edu>.
+   Heavily modified by David MacKenzie <djm@gnu.ai.mit.edu>.  */
 
-static const char *version_string = "GNU time version 1.6";
+extern const char *version_string;
 
+#include "wait.h"
 #include <stdio.h>
-#include <sys/types.h>
 #include <sys/param.h>		/* For getpagesize, maybe.  */
-#ifndef SIGINT			/* Thank you, OSF/1.  */
 #include <signal.h>
-#endif
 #include <errno.h>
 #ifndef errno
 extern int errno;
@@ -40,22 +39,63 @@ void error PARAMS((int status, int errnum, char *message, ...));
 static void usage PARAMS((FILE *, int));
 
 /* A Pointer to a signal handler.  */
-typedef RETSIGTYPE (*sighand_t) ();
+typedef RETSIGTYPE (*sighandler) ();
 
 /* msec = milliseconds = 1/1,000 (1*10e-3) second.
    usec = microseconds = 1/1,000,000 (1*10e-6) second.  */
 
-/* The number of milliseconds in one `tick' used by the `rusage' structure.
+/* Systems known to fill in the average resident set size fields:
+   SunOS 4.1.3 (m68k and sparc)
+   Mt. Xinu 4.3BSD on HP9000/300 (m68k)
+   Ultrix 4.4 (mips)
+   IBM ACIS 4.3BSD (rt)
+   Sony NEWS-OS 4.1C (m68k)
 
-   At least one Unix man page has an error, as it reports
-   units for average sizes as kb*sec.  Judging by the output
-   of `time' (all versions), this looks like it should be kb*ticks.
+   Systems known to not fill them in:
+   OSF/1 1.3 (alpha)
+   BSD/386 1.1 (anything derived from NET-2)
+   NetBSD 1.0 (4.4BSD-derived)
+   Irix 5.2 (R4000)
+   Solaris 2.3
+   Linux 1.0
 
-   1 tick = 20 msec = 1/50 sec.  */
-#define ONETICK (20)
+   It doesn't matter how many clock ticks/second there are on
+   systems that don't fill in those fields.
 
-/* Convert m milliseconds to ticks.  */
-#define MSEC_TO_TICKS(m) ((m) / ONETICK)
+   If the avgresident (%t) we print is less than a power of 2 away from
+   the maxresident (%M), then we likely are using the right number.
+   Another good check is comparing the average text size with the
+   output of `size' on the executable.
+
+   According to the SunOS manual, there are 50 ticks/sec on the sun3
+   and 100 on the sun4.
+
+   Some manuals have an apparent error, claiming that units for average
+   sizes are kb*sec.  Judging by the contents of `struct rusage', it
+   looks like it should be kb*ticks, like on SunOS.  Ticks/sec seems
+   to be (empirically):
+   50 Mt. Xinu
+   250 Ultrix (mips)
+   50 ACIS
+   100 NEWS-OS
+
+   sysconf(_SC_CLK_TCK) is *unrelated*.  */
+
+#if defined(sun3) || defined(hp300) || defined(ibm032)
+#define TICKS_PER_SEC 50
+#endif
+#if defined(mips)
+#define TICKS_PER_SEC 250
+#endif
+#ifndef TICKS_PER_SEC
+#define TICKS_PER_SEC 100
+#endif
+
+/* The number of milliseconds in one `tick' used by the `rusage' structure.  */
+#define MSEC_PER_TICK (1000 / TICKS_PER_SEC)
+
+/* Return the number of clock ticks that occur in M milliseconds.  */
+#define MSEC_TO_TICKS(m) ((m) / MSEC_PER_TICK)
 
 typedef enum {false, true} boolean;
 
@@ -287,8 +327,15 @@ summarize (fp, fmt, command, resp)
   unsigned long r;		/* Elapsed real milliseconds.  */
   unsigned long v;		/* Elapsed virtual (CPU) milliseconds.  */
 
-  if (resp->exitstatus)
-    fprintf (fp, "Command had non-zero exit status %d\n", resp->exitstatus);
+  if (WIFSTOPPED (resp->waitstatus))
+    fprintf (fp, "Command stopped by signal %d\n",
+	     WSTOPSIG (resp->waitstatus));
+  else if (WIFSIGNALED (resp->waitstatus))
+    fprintf (fp, "Command terminated by signal %d\n",
+	     WTERMSIG (resp->waitstatus));
+  else if (WIFEXITED (resp->waitstatus) && WEXITSTATUS (resp->waitstatus))
+    fprintf (fp, "Command exited with non-zero status %d\n",
+	     WEXITSTATUS (resp->waitstatus));
 
   /* Convert all times to milliseconds.  Occasionally, one of these values
      comes out as zero.  Dividing by zero causes problems, so we first
@@ -313,7 +360,7 @@ summarize (fp, fmt, command, resp)
 	    case 'C':		/* The command that got timed.  */
 	      fprintargv (fp, command, " ");
 	      break;
-	    case 'D':		/* Total unshared data size.  */
+	    case 'D':		/* Average unshared data size.  */
 	      fprintf (fp, "%lu",
 		       MSEC_TO_TICKS (v) == 0 ? 0 :
 		       ptok ((UL) resp->ru.ru_idrss) / MSEC_TO_TICKS (v) +
@@ -392,7 +439,7 @@ summarize (fp, fmt, command, resp)
 	    case 'k':		/* Signals delivered.  */
 	      fprintf (fp, "%ld", resp->ru.ru_nsignals);
 	      break;
-	    case 'p':		/* Stack segment.  */
+	    case 'p':		/* Average stack segment.  */
 	      fprintf (fp, "%lu",
 		       MSEC_TO_TICKS (v) == 0 ? 0 :
 		       ptok ((UL) resp->ru.ru_isrss) / MSEC_TO_TICKS (v));
@@ -412,7 +459,7 @@ summarize (fp, fmt, command, resp)
 	      fprintf (fp, "%ld", resp->ru.ru_nvcsw);
 	      break;
 	    case 'x':		/* Exit status.  */
-	      fprintf (fp, "%d", resp->exitstatus);
+	      fprintf (fp, "%d", WEXITSTATUS (resp->waitstatus));
 	      break;
 	    case '\0':
 	      putc ('?', fp);
@@ -549,11 +596,11 @@ getargs (argc, argv)
 
 static void
 run_command (cmd, resp)
-     const char **cmd;
+     char *const *cmd;
      RESUSE *resp;
 {
   pid_t pid;			/* Pid of child.  */
-  sighand_t interrupt_signal, quit_signal;
+  sighandler interrupt_signal, quit_signal;
 
   resuse_start (resp);
 
@@ -592,7 +639,14 @@ main (argc, argv)
   command_line = getargs (argc, argv);
   run_command (command_line, &res);
   summarize (outfp, output_format, command_line, &res);
-  exit (res.exitstatus);	/* Exit with child's status.  */
+  fflush (outfp);
+
+  if (WIFSTOPPED (res.waitstatus))
+    exit (WSTOPSIG (res.waitstatus));
+  else if (WIFSIGNALED (res.waitstatus))
+    exit (WTERMSIG (res.waitstatus));
+  else if (WIFEXITED (res.waitstatus))
+    exit (WEXITSTATUS (res.waitstatus));
 }
 
 static void
